package com.rjconsultores.ventaboletos.layouts;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

import org.jrimum.texgit.Record;

import com.rjconsultores.ventaboletos.blocos.CabecalhoRemessa;
import com.rjconsultores.ventaboletos.blocos.CabecalhoRetorno;
import com.rjconsultores.ventaboletos.blocos.DadosBoleto;
import com.rjconsultores.ventaboletos.blocos.DetalheObrigatorio;
import com.rjconsultores.ventaboletos.blocos.DetalheRetorno;
import com.rjconsultores.ventaboletos.blocos.RateioDeCredito;
import com.rjconsultores.ventaboletos.blocos.RodapeRemessa;
import com.rjconsultores.ventaboletos.blocos.RodapeRetorno;
import com.rjconsultores.ventaboletos.blocos.santander.CabecalhoRemessaSantander;
import com.rjconsultores.ventaboletos.blocos.santander.DetalheObrigatorioSantander;
import com.rjconsultores.ventaboletos.enuns.BancoLayout;
import com.rjconsultores.ventaboletos.utils.FormataUtil;

public class SantanderFlatFile extends AbstractFlatFile implements RemessaInterface {
	
	private static BigDecimal valorTotal = BigDecimal.ZERO;
	private static Integer quantidadeRegistro = 0;

	public SantanderFlatFile() {
		super(BancoLayout.SANTANDER_400_Envio);
		valorTotal = BigDecimal.ZERO;
	    quantidadeRegistro = 0;
	}
	
	@Override
	protected void createHeader(CabecalhoRemessa cabecalhoRemessaParam) {  
        
		Record header = getFlatFile().createRecord("Header");  
		
		CabecalhoRemessaSantander cabecalhoRemessa = (CabecalhoRemessaSantander) cabecalhoRemessaParam;
		
        header.setValue("CodigoDaEmpresa", FormataUtil.formatarValorPorTamanho(cabecalhoRemessa.getCodigoEmpresa().toString(), 20));  
        header.setValue("NomeCedente", FormataUtil.formatarValorPorTamanho(cabecalhoRemessa.getNomeEmpresa(), 30));  
        header.setValue("CodigoTransmissao", FormataUtil.formatarValorPorTamanho(cabecalhoRemessa.getCodigoTransmissao().toString(), 20));  
        header.setValue("DataGravacao", FormataUtil.formataDataToString(cabecalhoRemessa.getDataGravacao(), "000000", "ddMMyy"));   
        header.setValue("NumeroSequencialRemessa", FormataUtil.formatarValorPorTamanho(cabecalhoRemessa.getNumeroSequencialRemessa().toString(), 7));  
        header.setValue("NumeroSequencialRegistro", FormataUtil.formatarValorPorTamanho(cabecalhoRemessa.getNumeroSequencialRegistro().toString(), 6)); 
        
        quantidadeRegistro = quantidadeRegistro + 1;
        
        getFlatFile().addRecord(header);   
    } 
	
	@Override
	protected void createTransacaoTitulos(DetalheObrigatorio tituloParam) {  
        
		Record transacaoTitulos = getFlatFile().createRecord("TransacaoTitulo");  
		
		DetalheObrigatorioSantander titulo = (DetalheObrigatorioSantander) tituloParam;
		
		transacaoTitulos.setValue("TipoInscricao", FormataUtil.formatarValorPorTamanho(titulo.getTipoInscricao().toString(), 2));
		transacaoTitulos.setValue("NumeroInscricao", FormataUtil.formatarValorPorTamanho(titulo.getNumeroInscricaoSacado().toString(), 14));
		transacaoTitulos.setValue("CodigoTransmissao", FormataUtil.formatarValorPorTamanho(titulo.getCodigoTransmissao().toString(), 20));
		transacaoTitulos.setValue("NumControleDoParticipante", FormataUtil.formatarValorPorTamanho(titulo.getNumControleDoParticipante(), 25));
		transacaoTitulos.setValue("NossoNumeroComDigito", FormataUtil.formatarValorPorTamanho(titulo.getNossoNumeroComDigito(), 8));
		transacaoTitulos.setValue("Carteira", FormataUtil.formatarValorPorTamanho(titulo.getCarteira(), 3));
		transacaoTitulos.setValue("MultaPercentual", titulo.getPercentualMulta());
		transacaoTitulos.setValue("JurosDeMora", FormataUtil.formatarValorPorTamanho(titulo.getValorAtraso(), 13));
		transacaoTitulos.setValue("Agencia", FormataUtil.formatarValorPorTamanho(titulo.getAgencia(), 4));
		transacaoTitulos.setValue("DVAgencia", FormataUtil.formatarValorPorTamanho(titulo.getDvAgencia(), 1));
		transacaoTitulos.setValue("CodigoEmpresa", FormataUtil.formatarValorPorTamanho(titulo.getCodigoEmpresa(), 7));
		transacaoTitulos.setValue("ContaCorrente", FormataUtil.formatarValorPorTamanho(titulo.getContacorrente(), 7)); 
		transacaoTitulos.setValue("DvConta", FormataUtil.formatarValorPorTamanho(titulo.getDvConta(), 1));
		transacaoTitulos.setValue("ContaCobrancaBeneficiario", FormataUtil.formatarValorPorTamanho(titulo.getContacorrente(), 8));
		transacaoTitulos.setValue("DvContaCobrancaBeneficiario", FormataUtil.formatarValorPorTamanho(titulo.getDvConta(), 1));
		
		transacaoTitulos.setValue("Filler1", FormataUtil.formatarValorPorTamanho("0", 8));
		
		transacaoTitulos.setValue("QtdPagamento", FormataUtil.formatarValorPorTamanho(titulo.getQtdPagamento().toString(), 25));
		transacaoTitulos.setValue("NumeroDoDocumento", FormataUtil.formatarValorPorTamanho(titulo.getNumeroDoDocumento(), 10));
		transacaoTitulos.setValue("Vencimento", FormataUtil.formataDataToString(titulo.getVencimento(), "000000", "ddMMyy"));
		transacaoTitulos.setValue("Valor", titulo.getValor());
		transacaoTitulos.setValue("EspecieDeTitulo", FormataUtil.formatarValorPorTamanho(titulo.getEspecieDeTitulo(), 2));
		transacaoTitulos.setValue("NumeroInscricaoPagador", FormataUtil.formatarValorPorTamanho(titulo.getNumeroInscricao().toString(), 14));
		transacaoTitulos.setValue("AgenciaCobradora", FormataUtil.formatarValorPorTamanho(titulo.getAgenciaCobradora(), 4));
		transacaoTitulos.setValue("DVAgenciaCobradora", FormataUtil.formatarValorPorTamanho(titulo.getDvAgencia(), 1));

		transacaoTitulos.setValue("Emissao", FormataUtil.formataDataToString(titulo.getEmissao(), "000000", "ddMMyy"));
		transacaoTitulos.setValue("Instrucao1", FormataUtil.formatarValorPorTamanho(titulo.getInstrucao1(), 2));
		transacaoTitulos.setValue("Instrucao2", FormataUtil.formatarValorPorTamanho(titulo.getInstrucao2(), 2));
		transacaoTitulos.setValue("JurosDeMora", titulo.getValorAtraso());
//		transacaoTitulos.setValue("DataLimiteDesconto", FormataUtil.formataData(titulo.getDataLimiteDesconto()));
		transacaoTitulos.setValue("ValorDesconto", titulo.getValorDesconto());
		transacaoTitulos.setValue("ValorIOF", titulo.getValorIOF());
		transacaoTitulos.setValue("ValorAbatimentoConcedido", titulo.getValorAbatimentoConcedido());
		transacaoTitulos.setValue("Mensagem1", FormataUtil.formatarValorPorTamanho(titulo.getMensagem1(), 12));
		transacaoTitulos.setValue("NomeSacado", FormataUtil.formatarValorPorTamanho(titulo.getNomeSacado(), 40));
		transacaoTitulos.setValue("EnderecoSacado", FormataUtil.formatarValorPorTamanho(titulo.getEnderecoSacado(), 40));
		transacaoTitulos.setValue("BairroSacado", FormataUtil.formatarValorPorTamanho(titulo.getBairroSacado(), 12));
		transacaoTitulos.setValue("CepSacado", FormataUtil.formatarValorPorTamanho(titulo.getCepSacado().toString(), 5));
		transacaoTitulos.setValue("ComplementoCepSacado", FormataUtil.formatarValorPorTamanho(titulo.getComplementoCepSacado().toString(), 3));
		transacaoTitulos.setValue("Cidade", FormataUtil.formatarValorPorTamanho(titulo.getCidade(), 15));
		transacaoTitulos.setValue("Estado", FormataUtil.formatarValorPorTamanho(titulo.getEstado(), 2));
		transacaoTitulos.setValue("SacadorAvalista", FormataUtil.formatarValorPorTamanho(titulo.getSacadorAvalista(), 30));
		transacaoTitulos.setValue("NumeroSequencialRegistro", FormataUtil.formatarValorPorTamanho(titulo.getNumeroSequencialRegistro().toString(), 6));
		transacaoTitulos.setValue("Complemento", FormataUtil.formatarValorPorTamanho(titulo.getContacorrente().substring(titulo.getContacorrente().length()-1, titulo.getContacorrente().length()).concat(titulo.getDvConta()),2));

		valorTotal = valorTotal.add(titulo.getValor());
		quantidadeRegistro = quantidadeRegistro + 1;
	
		DadosBoleto dadosBoleto = titulo.getDadosBoleto();
		
		try{
			if(dadosBoleto != null){
				transacaoTitulos.addInnerRecord(createDadosBoletos(dadosBoleto));
			}
		}catch(UnsupportedOperationException e){		}
		
		RateioDeCredito rateioDeCredito = titulo.getRateioDeCredito();
		
		try{
			if(dadosBoleto != null){
				transacaoTitulos.addInnerRecord(createRateioDeCredito(rateioDeCredito));
			}
		}catch(UnsupportedOperationException e){		}
        
		getFlatFile().addRecord(transacaoTitulos); 
    } 
	
	@Override
	protected Record createDadosBoletos(DadosBoleto boleto) {  
		throw new UnsupportedOperationException("Não necessário...");
	}
	
	@Override
	protected Record createRateioDeCredito(RateioDeCredito rateioDeCredito) {
		throw new UnsupportedOperationException("Não necessário...");
	}
	
	@Override
	protected void createTrailler(RodapeRemessa trailler){
		
		Record trailer = getFlatFile().createRecord("Trailler");  
		
        trailer.setValue("NumeroSequencialRegistro", FormataUtil.formatarValorPorTamanho(trailler.getNumeroSequencialRegistro().toString(), 6)); 
        trailer.setValue("QuantidadeRegistro",quantidadeRegistro+1); 
        trailer.setValue("ValorTotal", valorTotal); 
        
        getFlatFile().addRecord(trailer); 

	}

	@Override
	protected CabecalhoRetorno readHeader(Record header) {
		throw new UnsupportedOperationException("Somente classe de Retorno.");
	}

	@Override
	protected List<DetalheRetorno> readTransacaoTitulos(Collection<Record> registrosDeTransacoes) {
		throw new UnsupportedOperationException("Somente classe de Retorno.");
	}

	@Override
	protected RodapeRetorno readTrailler(Record trailler) {
		throw new UnsupportedOperationException("Somente classe de Retorno.");
	}
}
